<?php

namespace FM\Controller;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use FM\Service\ListagemVistoriasService;
use FM\Traits\TResponse;

class ListagemVistoriasController
{
    use TResponse;

    protected $ListagemVistoriasService;

    public function __construct($ListagemVistoriasService)
    {
        $this->ListagemVistoriasService = $ListagemVistoriasService;
    }

    public function getVistoriasListadas(Request $request)
    {
        try {
            $data = $this->ListagemVistoriasService->getVistoriasListadas($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getFotosVistorias(Request $request)
    {
        try {
            $data = $this->ListagemVistoriasService->getFotosVistorias($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function excluirFotoVistorias(Request $request)
    {
        try {
            $data = $this->ListagemVistoriasService->excluirFotoVistorias($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCardsSurveys(DTO\Request $request, DTO\Response $response)
    {
        $parametro = $request->getParameters();

        $variavel = $this->ListagemVistoriasService->getCardsSurveys($parametro['filter'], $parametro['usersIds']);
        $dtsVal = new DataSet('cardsSurveys', $variavel);
        $response->addDataSet($dtsVal);
    }

    public function getDetailsSurvey(DTO\Request $request, DTO\Response $response)
    {

        $parametro = $request->getParameters();

        $variavel = $this->ListagemVistoriasService->getDetailsSurvey($parametro);
        $dtsVal = new DataSet('detailsSurvey', $variavel);
        $response->addDataSet($dtsVal);
    }

    // Funções da tela de Detalhes da Vistoria

    public function getTask(DTO\Request $request, DTO\Response $response)
    {
        $parametro = $request->getParameters();

        $variavel = $this->ListagemVistoriasService->getTask($parametro);
        $dtsVal = new DataSet('criterias', [$variavel]);
        $response->addDataSet($dtsVal);
    }

    public function upSurveyCriterias(DTO\Request $request, DTO\Response $response)
    {

        try {
            $parametro = $request->getParameters();

            $variavel = $this->ListagemVistoriasService->upSurveyCriterias($parametro[0], $parametro[1]);
            $dtsVal = new DataSet('upCriterias', [$variavel]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
