<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class ListagemVistoriasFactory
{

    use TQuery;
    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager    = $entityManager;
    }

    public function getVistoriasListadas($listaOperadoes, $filtros)
    {
        $sql = "SELECT GCHK.NRGERENCIAMENTOCHK, GCHK.NMGERENCIAMENTOCHK, GCHK.NRLOCALCLIENTE, GCHK.CDOPERADOR, 
                        GCHK.IDSITUACAOGERCHK, GCHK.NRSEQCONTRATO, CT.NRCONTRATOFCT, C.NMFANTCLIE,
                        TO_CHAR(GCHK.DTHORAAGEND, 'DD/MM') DTAGEND,  TO_CHAR(GCHK.DTHORAAGEND, 'HH24:MI') HORAAGEND,
                        FUNC.NMOPERADOR, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI, LCP.NMLOCALCLIENTE AS NMLOCALPAI, 
                        CA.IDSITUACAOAGEND, FUNCOP.NMOPERADOR AS SUPERVISOR, CA.NMCHECKLISTAGEND,
                DECODE(CA.IDSITUACAOAGEND, '4', 
                        CASE GCHK.IDSITUACAOGERCHK 
                                WHEN '1' THEN 'Aguardando Vistoria'
                                WHEN '2' THEN 'Em andamento'
                                WHEN '3' THEN 'Avaliado'
                        ELSE NULL END, DECODE(GCHK.IDSITUACAOGERCHK, '4', 'Cancelado', 'Tarefa Pendente')) AS SITUACAO
                FROM FCM_GERENCIAMENTOCHK GCHK
                INNER JOIN FCM_LOCALCLI LC 
                    ON LC.NRLOCALCLIENTE = GCHK.NRLOCALCLIENTE
                INNER JOIN OPERADOR FUNC 
                    ON FUNC.CDOPERADOR = GCHK.CDOPERADOR
                INNER JOIN FCM_LOCALCLI LCP 
                    ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                INNER JOIN FCM_CHECKLISTAGEND CA 
                    ON CA.NRCHECKLISTAGEND = GCHK.NRCHECKLISTAGEND
                LEFT JOIN FCT_CONTRATO CT
                    ON CT.NRSEQCONTRATO = GCHK.NRSEQCONTRATO
                LEFT JOIN CLIENTE C
                    ON C.CDCLIENTE = LC.CDCLIENTE
                INNER JOIN OPERADOR FUNCOP
                    ON FUNCOP.CDOPERADOR = CA.CDOPERADOR
                WHERE GCHK.CDOPERADOR 
                    IN ({$listaOperadoes})" . $filtros . "
                ORDER BY GCHK.DTHORAAGEND";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFotosVistorias($parametros)
    {

        $sql = "SELECT FCM_FOTOTAREFAGERCHK.NRFOTOTAREFAGERCHK, FCM_FOTOTAREFAGERCHK.DSFOTOTAREFAGERCHK, FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK
                    FROM FCM_FOTOTAREFAGERCHK
                    JOIN FCM_TAREFAGERCHK 
                    ON FCM_TAREFAGERCHK.NRTAREFAGERCHK = FCM_FOTOTAREFAGERCHK.NRTAREFAGERCHK
                    JOIN FCM_GERENCIAMENTOCHK 
                    ON FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK = FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK
                    WHERE FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function excluirFotoVistorias($parametros)
    {

        $sql = "DELETE FROM FCM_FOTOTAREFAGERCHK
                    WHERE NRFOTOTAREFAGERCHK = :NRFOTOTAREFAGERCHK";

        $this->connection->executeStatement($sql, $parametros);
    }

    // Funções da Tela de Listagem de Vistorias
    public function getPlaceSurvey()
    {

        $sql = "SELECT DISTINCT LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTE
                  FROM FCM_LOCALCLI LC
                  JOIN FCM_GERENCIAMENTOCHK GCHK
                  ON LC.NRLOCALCLIENTE = GCHK.NRLOCALCLIENTE";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getEmployee($usersIds)
    {

        $sql = "SELECT DISTINCT FUNC.NMFUNCIONARIO, FUNC.NRFUNCIONARIO
                  FROM FCM_FUNCIONARIO FUNC
                  JOIN FCM_GERENCIAMENTOCHK GCHK
                  ON FUNC.NRFUNCIONARIO = GCHK.NRFUNCIONARIO
                 WHERE FUNC.NRFUNCIONARIO IN ({$usersIds})";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getSituation()
    {

        $sql = "SELECT DISTINCT FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK AS NRIDSITUACAOAGEND,
                CASE FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK WHEN '1' THEN 'Aguardando vistoria'
                   WHEN '2' THEN 'Em andamento'
                   WHEN '3' THEN 'Avaliado'
                   WHEN '4' THEN 'Cancelado'
                   ELSE NULL END AS IDSITUACAOGERCHK
                   FROM FCM_GERENCIAMENTOCHK";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function cardsSurveys($parametro, $usersIds)
    {
        $sql = "SELECT GCHK.NRGERENCIAMENTOCHK, GCHK.NMGERENCIAMENTOCHK, GCHK.NRLOCALCLIENTE, GCHK.CDOPERADOR, GCHK.IDSITUACAOGERCHK,
                  TO_CHAR(GCHK.DTHORAAGEND, 'DD/MM HH24:MI')DTHORAAGEND, FUNC.NMOPERADOR, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI, LCP.NMLOCALCLIENTE AS NMLOCALPAI, CA.IDSITUACAOAGEND,
                      DECODE(CA.IDSITUACAOAGEND, '4', CASE GCHK.IDSITUACAOGERCHK WHEN '1' THEN 'Aguardando Vistoria'
                                        WHEN '2' THEN 'Em andamento'
                                        WHEN '3' THEN 'Avaliado'
                                        ELSE NULL END, DECODE(GCHK.IDSITUACAOGERCHK, '4', 'Cancelado', 'Tarefa Pendente')) AS IDSITUACAOGERCHK
                          FROM FCM_GERENCIAMENTOCHK GCHK
                          INNER JOIN FCM_LOCALCLI LC
                          ON LC.NRLOCALCLIENTE = GCHK.NRLOCALCLIENTE
                          INNER JOIN OPERADOR FUNC
                          ON FUNC.CDOPERADOR = GCHK.CDOPERADOR
                          INNER JOIN FCM_LOCALCLI LCP
                          ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                          INNER JOIN FCM_CHECKLISTAGEND CA
                          ON CA.NRCHECKLISTAGEND = GCHK.NRCHECKLISTAGEND" . $parametro . "
                          AND GCHK.CDOPERADOR IN ({$usersIds})
                          ORDER BY DTHORAAGEND";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    // Funções da tela de Detlahes da Vistoria

    // Select para trazer as atividades
    public function getActivities($parametros)
    {

        $sql = "SELECT DISTINCT FCM_ATIVIDADECHK.NRATIVIDADECHK, FCM_ATIVIDADE.NMATIVIDADE
                  FROM FCM_ATIVIDADECHK
                  JOIN FCM_ATIVIDADE
                    ON FCM_ATIVIDADE.NRATIVIDADE = FCM_ATIVIDADECHK.NRATIVIDADE
                  JOIN FCM_TAREFACHK
                    ON FCM_TAREFACHK.NRATIVIDADECHK = FCM_ATIVIDADECHK.NRATIVIDADECHK
                    AND NRTAREFACHK IN (SELECT NRTAREFACHK
                                          FROM FCM_TAREFAGERCHK
                                         WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK)";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    // Select para trazer a tarefa (Onde estarão as observações)
    // Caso esse select traga mais de uma observação, pode concatenar as observações,
    // colcoando duas quebras de linha entre elas)
    public function getTask($parametros)
    {

        $sql = "SELECT FCM_TAREFAGERCHK.NRTAREFAGERCHK, FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK, 
                FCM_TAREFAGERCHK.DSOBSTAREFAGERCHK, FCM_GERENCIAMENTOCHK.IDSITUACAOGERCHK, FCM_CHECKLISTAGEND.IDSITUACAOAGEND,
                FCM_GERENCIAMENTOCHK.IDOBRIGAOBSERVACAO
                  FROM FCM_TAREFAGERCHK
                  JOIN FCM_TAREFACHK
                  ON FCM_TAREFACHK.NRTAREFACHK = FCM_TAREFAGERCHK.NRTAREFACHK
                  JOIN FCM_GERENCIAMENTOCHK
                  ON FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK = FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK
                  JOIN FCM_CHECKLISTAGEND
                  ON FCM_CHECKLISTAGEND.NRCHECKLISTAGEND = FCM_GERENCIAMENTOCHK.NRCHECKLISTAGEND
                  WHERE FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
                  AND FCM_TAREFACHK.NRATIVIDADECHK = :NRATIVIDADECHK";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    // Select para trazer os critérios
    public function getCriteria($parametros)
    {

        $sql = "SELECT NRCRITERIOTARGERCHK, NMCRITERIOTARGERCHK, IDMARCADO
                  FROM FCM_CRITERIOTARGERCHK
                 WHERE NRTAREFAGERCHK = :NRTAREFAGERCHK";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function upCriterias($parametros, $paramQuery)
    {

        $sql = "UPDATE FCM_CRITERIOTARGERCHK
                  SET IDMARCADO = :IDMARCADO,
                    DTULTATU = SYSDATE
                  WHERE NRCRITERIOTARGERCHK IN (SELECT NRCRITERIOTARGERCHK
                                                    FROM FCM_CRITERIOTARGERCHK
                                                   WHERE NRTAREFAGERCHK IN (SELECT FCM_TAREFAGERCHK.NRTAREFAGERCHK
                                                    FROM FCM_TAREFAGERCHK
                                                    JOIN FCM_TAREFACHK
                                                      ON FCM_TAREFACHK.NRTAREFACHK = FCM_TAREFAGERCHK.NRTAREFACHK
                                                   WHERE FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
                                                     AND FCM_TAREFACHK.NRATIVIDADECHK = :NRATIVIDADECHK))" . $paramQuery;

        $this->connection->executeStatement($sql, $parametros);
    }

    public function upObservation($parametros)
    {

        $sql = "UPDATE FCM_TAREFAGERCHK
                SET DSOBSTAREFAGERCHK = :DSOBSTAREFAGERCHK
                    WHERE NRTAREFAGERCHK IN (SELECT FCM_TAREFAGERCHK.NRTAREFAGERCHK
                                  FROM FCM_TAREFAGERCHK
                                  JOIN FCM_TAREFACHK
                                    ON FCM_TAREFACHK.NRTAREFACHK = FCM_TAREFAGERCHK.NRTAREFACHK
                                 WHERE FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK
                                   AND FCM_TAREFACHK.NRATIVIDADECHK = :NRATIVIDADECHK)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function upStatusSurvey($parametros)
    {

        $sql = "UPDATE FCM_GERENCIAMENTOCHK
                SET IDSITUACAOGERCHK = :IDSITUACAOGERCHK,
                DTULTATU = SYSDATE
                  WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";


        $this->connection->executeStatement($sql, $parametros);
    }
}
