import { Utilities } from '@/util/Utilities';
import {
    Button, Checkbox, DateRange, FileInput, Form, LoadingService, Modal, ModalService, SelectMultiple, Textarea, TextInput,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import {
    IterableCarousel, IterableComponentRender,
} from '@zeedhi/teknisa-components-common';
import FileUpload from './FileUpload';

export class Fm00001TarefasController {
    public desabilitarFinalizacaoTarefa: boolean = false;

    public desabilitarChk: boolean = false;

    public visibilidadePaginaInicial: boolean = true;

    public visibilidadeDetalhamento: boolean = false;

    public visibilidadeReportar: boolean = false;

    public visibilidadeCarroselImagens: boolean = false;

    public statusTarefa = '';

    public nomeTarefa = '';

    public localTarefa = '';

    public dataTarefa = '';

    public iniciada: boolean = false;

    public modalFiltrarMinhasTarefas!: Modal;

    public modalAnexarImagem!: Modal;

    private nrChecklistAgend = '';

    private modoAberturaReportar: boolean = false;

    public async criarModais() {
        this.modalFiltrarMinhasTarefas = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00001_000_tarefas/modal/fm_00001_000_filtrar_minhas_tarefas' });
        this.modalAnexarImagem = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00001_000_tarefas/modal/fm_00001_000_anexar_imagens' });
    }

    public destruirModais() {
        ModalService.delete(this.modalFiltrarMinhasTarefas);
        ModalService.delete(this.modalAnexarImagem);
    }

    public abrirModalFiltrar() {
        this.modalFiltrarMinhasTarefas.show();
    }

    public fecharModalFiltrar() {
        this.modalFiltrarMinhasTarefas.hide();
    }

    public abrirModalAnexar() {
        this.modalAnexarImagem.show();
    }

    public fecharModalAnexar() {
        this.modalAnexarImagem.hide();
    }

    public exibirImagemAnexada({ component }: any) {
        const { datasource } = Metadata.getInstance<IterableCarousel>('carousel-imagens').iterable;
        const imagens: any = [];

        if (component.value.length) {
            this.visibilidadeCarroselImagens = true;
            component.value.forEach((elemento: any, posicao: any) => {
                imagens.push({
                    id: posicao,
                    src: window.URL.createObjectURL(elemento),
                });
            });
        } else {
            this.visibilidadeCarroselImagens = false;
        }

        datasource.updateData(imagens);
    }

    public async anexarImagens() {
        const imagensConvertidas = await FileUpload.converterImagens('file-input-anexar-imagem');
        const fileInput = Metadata.getInstance<FileInput>('file-input-anexar-imagem');

        if (fileInput.value) {
            Http.post('/anexarImagensTarefas', {
                IMAGENS: imagensConvertidas,
                NRCHECKLISTAGEND: Metadata.getInstance<TextInput>('nrcheck').value,
            }).then(() => {
                Utilities.showSuccessMessage();
                this.fecharModalAnexar();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.alertMessage('Nenhuma imagem foi anexada', 'error');
        }
    }

    public carregarCardsMinhasTarefas({ component }: any) {
        component.datasource.setFilter({
            TIPO: 1,
        });
    }

    public filtrarCards() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas');

        iterable.datasource.setFilter({
            CDCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-cliente').value,
            NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('filtro-local').value,
            NRSEQCONTRATO: Metadata.getInstance<SelectMultiple>('filtro-contratos').value,
            NRCHECKLISTAGEND: Metadata.getInstance<SelectMultiple>('filtro-tarefas').value,
            DTHORAGEND: Metadata.getInstance<DateRange>('data').value,
            TIPO: 2,
        }).finally(() => {
            this.fecharModalFiltrar();
        });
    }

    public limparFiltros() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas');

        Metadata.getInstance<SelectMultiple>('filtro-cliente').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-local').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-contratos').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-tarefas').value = null;
        Metadata.getInstance<DateRange>('data').value = null;

        this.carregarCardsMinhasTarefas({ component: iterable });
    }

    public clickCard({ component }: any) {
        this.visibilidadePaginaInicial = false;
        this.visibilidadeDetalhamento = !this.visibilidadePaginaInicial;

        this.montarInformacoesComplementares(component.name.split('_')[1]);
    }

    public retornarPaginaInicial() {
        this.visibilidadePaginaInicial = true;
        this.visibilidadeDetalhamento = !this.visibilidadePaginaInicial;
        this.limparCampos();
    }

    public limparCampos() {
        Metadata.getInstance<Textarea>('box-observation').value = null;
    }

    // Funções da tela de Detalhamento
    public desabilitarCheckbox() {
        const status = Metadata.getInstance<TextInput>('idsituacao');
        Http.get('/qtdTarefasAtivas').then((response: IDictionary) => {
            if (response.data[0].TAREFAATIVA === 1 && status.value !== '2') {
                this.desabilitarChk = true;
            } else if (response.data[0].TAREFAATIVA === 0) {
                this.desabilitarChk = false;
            } else {
                this.desabilitarChk = false;
            }
        });
    }

    public alterarBotao({ component }: IEventParam<Button>) {
        const status = Metadata.getInstance<TextInput>('idsituacao');
        if (status.value) {
            LoadingService.show();
            Http.get('/qtdTarefasAtivas').then((response: IDictionary) => {
                if (response.data[0].TAREFAATIVA === 0 && status.value !== '2') {
                    this.estilizarBotao({ component }, 'Iniciar');
                    component.disabled = false;
                } else if (response.data[0].TAREFAATIVA === 1 && status.value === '2') {
                    this.estilizarBotao({ component }, 'Finalizar');
                    component.disabled = false;
                } else if (response.data[0].TAREFAATIVA === 1 && status.value !== '2') {
                    this.estilizarBotao({ component }, 'Iniciar');
                    component.disabled = true;
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public estilizarBotao({ component }: IEventParam<Button>, tipo: any) {
        if (tipo === 'Iniciar') {
            component.label = 'Iniciar Tarefa';
            component.iconName = 'mdi-chevron-right';
            component.color = 'primary';
        } else {
            component.label = 'Finalizar Tarefa';
            component.iconName = 'mdi-chevron-right';
            component.color = 'success';
        }
    }

    public iniciarTarefaAutomaticamente() {
        const botao = Metadata.getInstance<Button>('botao-iniciar-finalizar-tarefa');
        const nr = Metadata.getInstance<TextInput>('nrcheck').value;
        const status = Metadata.getInstance<TextInput>('idsituacao').value;

        if (!this.iniciada && status !== '2') {
            LoadingService.show();
            Http.get('/qtdTarefasAtivas').then((response: IDictionary) => {
                if (response.data[0].TAREFAATIVA === 0) {
                    this.iniciarTarefa(nr, 'iniciar', '2');
                    this.estilizarBotao({ component: botao }, 'Finalizar');
                    botao.disabled = false;
                    this.statusTarefa = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
                    this.iniciada = true;
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public iniciarTarefa(nr: any, tipo: string, id: any) {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas');

        LoadingService.show();
        Http.post('/alterarStatusTarefa', {
            NRCHECKLISTAGEND: nr,
            TIPO: tipo,
            IDSITUACAOAGEND: id,
        }).then(() => {
            Utilities.alertMessage('Tarefa iniciada com sucesso!', 'success');
            iterable.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public iniciarOuFinalizar({ component }: any) {
        const nr = Metadata.getInstance<TextInput>('nrcheck').value;

        if (component.label === 'Iniciar Tarefa') {
            this.iniciarTarefa(nr, 'iniciar', '2');
                this.estilizarBotao({ component }, 'Finalizar');
                component.disabled = false;
                this.statusTarefa = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
        } else {
            this.statusTarefa = "<span  style='background-color: #dee2e6; color: green; font-size: 10px;'>C O N C L U Í D O</span>";
            this.finalizarTarefa(nr);
        }
    }

    public finalizarTarefa(nr: any) {
        const chkbox: IDictionary = [];
        const formObs = Metadata.getInstance<Form>('Form-observation');
        const cardsPrincipais = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas');

        Metadata.getInstance<IterableComponentRender>('iterable-cards-detalhes').datasource.data.forEach((itens: IDictionary) => {
            if (Metadata.getInstance<Checkbox>(`checkbox_${itens.NRTAREFAAGEND}`).value) {
                chkbox.push(itens.NRTAREFAAGEND);
            }
        });
        if (Utilities.validateForm(formObs)) {
            LoadingService.hide();
            Http.post('/finalizarTarefa', {
                NRCHECKLISTAGEND: nr,
                CHKBOX: chkbox,
                DSOBSTAREFAAGEND: Metadata.getInstance<Textarea>('box-observation').value,
            }).then(() => {
               Utilities.showSuccessMessage();
               this.visibilidadeDetalhamento = false;
               this.visibilidadePaginaInicial = true;
               this.limparCampos();
               cardsPrincipais.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
                this.iniciada = false;
            });
        }
    }

    public montarInformacoesComplementares(nr: any) {
        LoadingService.show();
        const { data } = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas').datasource;
        const observacao = Metadata.getInstance<Textarea>('box-observation');
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-cards-detalhes');
        const botao = Metadata.getInstance<Button>('botao-iniciar-finalizar-tarefa');

        const dados: any = data.find((row: any) => row.NRCHECKLISTAGEND === Number(nr));

        Metadata.getInstance<TextInput>('nrcheck').value = dados.NRCHECKLISTAGEND;
        Metadata.getInstance<TextInput>('idsituacao').value = dados.IDSITUACAOAGEND;
        this.nomeTarefa = dados.NMCHECKLISTAGEND;
        this.localTarefa = `${dados.NMLOCALCLIENTE} - ${dados.NMLOCALPAI}`;
        this.dataTarefa = `${dados.DTAGEND} ${dados.HORAAGEND}`;
        this.statusTarefa = this.estilizarStatus(dados.SITUACAO);

        this.alterarBotao({ component: botao });
        this.carregarItens({ component: iterable });
        this.desabilitarCheckbox();

        if (dados.IDOBRIGAOBSERVACAO !== 'S') {
            observacao.clearValidations();
        } else {
            observacao.addValidation('required', {});
        }

        LoadingService.hide();
    }

    public carregarItens({ component }: IEventParam<IterableComponentRender>) {
        const nr = Metadata.getInstance<TextInput>('nrcheck').value;
        component.datasource.setFilter({
            NRCHECKLISTAGEND: nr,
        });
    }

    public estilizarStatus(status: string) {
        if (status === 'Pendente') {
            return "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 10px;'>P E N D E N T E</span>";
        }
        if (status === 'Em andamento') {
            return "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
        }
        if (status === 'Atrasado') {
            return "<span  style='background-color: #dee2e6; color: red; font-size: 10px;'>A T R A S A D O</span>";
        }
        if (status === 'Concluído') {
            return "<span  style='background-color: #dee2e6; color: green; font-size: 10px;'>C O N C L U Í D O</span>";
        }
        return '';
    }

    // Funções da tela de Reportar
    public addTextArea({ component }: IEventParam<Button>) {
        const btOcupado = Metadata.getInstance<Button>('btOcupado');
        const btSemAcesso = Metadata.getInstance<Button>('btSemAcesso');
        const btEspacoLimpo = Metadata.getInstance<Button>('btEspacoLimpo');
        const btOutrosMotivos = Metadata.getInstance<Button>('btOutrosMotivos');
        const textA = Metadata.getInstance<Textarea>('textarea-justificativa');

        btOcupado.outline = !(component.label === btOcupado.label);
        btSemAcesso.outline = !(component.label === btSemAcesso.label);
        btEspacoLimpo.outline = !(component.label === btEspacoLimpo.label);
        btOutrosMotivos.outline = !(component.label === btOutrosMotivos.label);

        if (component.label === btOutrosMotivos.label) {
          textA.isVisible = true;
        } else {
          textA.isVisible = false;
        }
    }

    public abrirReportarBotaoExterno({ component }: IEventParam<Button>) {
        this.visibilidadePaginaInicial = false;
        this.visibilidadeReportar = !this.visibilidadePaginaInicial;
        const nr = component.name.split('_')[1];
        this.nrChecklistAgend = nr;
        this.modoAberturaReportar = true;
    }

    public abrirReportarBotaoInterno() {
        this.visibilidadeDetalhamento = false;
        this.visibilidadeReportar = !this.visibilidadeDetalhamento;
        this.nrChecklistAgend = Metadata.getInstance<TextInput>('nrcheck').value;
        this.modoAberturaReportar = false;
    }

    public limparTelaReportar() {
        Metadata.getInstance<Button>('btOcupado').outline = true;
        Metadata.getInstance<Button>('btSemAcesso').outline = true;
        Metadata.getInstance<Button>('btEspacoLimpo').outline = true;
        Metadata.getInstance<Button>('btOutrosMotivos').outline = true;
        Metadata.getInstance<Textarea>('textarea-justificativa').value = null;
    }

    public fecharPaginaReportar() {
        this.visibilidadeReportar = false;
        if (!this.modoAberturaReportar) {
            this.visibilidadeDetalhamento = true;
        } else {
            this.visibilidadePaginaInicial = true;
        }
        this.limparTelaReportar();
    }

    public reportar() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-minhas-tarefas');
        let justify = ['btOcupado', 'btSemAcesso', 'btEspacoLimpo', 'btOutrosMotivos'].filter((nameButton: string) => Metadata.getInstance<Button>(nameButton).outline === false)[0];

        if (justify === 'btOutrosMotivos') {
            justify = (Metadata.getInstance<Textarea>('textarea-justificativa').value.trim().replace(' ', '').length > 0) ? Metadata.getInstance<Textarea>('textarea-justificativa').value.trim() : justify;
        } else {
            justify = Metadata.getInstance<Button>(justify).label;
        }

        if (justify !== 'btOutrosMotivos') {
            Http.post('/alterarStatusTarefa', {
                NRCHECKLISTAGEND: this.nrChecklistAgend,
                TIPO: 'reportar',
                IDSITUACAOAGEND: 3,
                DSJUSTCANCELAMENTO: justify,
            }).then(() => {
                Utilities.showSuccessMessage();
                iterable.datasource.get();
                this.visibilidadeReportar = false;
                this.visibilidadePaginaInicial = true;
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        } else {
            Utilities.alertMessage('Favor preencher o motivo', 'error');
        }
    }
}
