/* eslint-disable */
import { Utilities } from '@/util/Utilities';
import {
    ModalService, Modal, IModal, DialogService, LoadingService,
} from '@zeedhi/common';
import {
    Router, Http, Loader, Metadata, Singleton,
} from '@zeedhi/core';
import {
    TekGrid,
} from '@zeedhi/teknisa-components-common';

@Singleton
export class QrCodeController {
    private qrCodeModal!: Modal;

    public url: String = '';

    public testeModal!: Modal;

    public dadosCard: any;

    public dadosSeguros: any = [];

    public telaVistoria = Loader.getInstance('ListagemVistoriasController');

    public telaTarefas = Loader.getInstance('Fm00001TarefasController');

    public decodeQrCode({ decodeString }: any) {
        this.url = decodeString;
        this.buscaOperador();
    }

    public buscaOperador() {
        LoadingService.show();
        Http.post('/buscaTarefasPorOperador', {
            ATIVO: 'S',
            NRLOCALCLIENTE: this.url,
            CDOPERADOR: window.localStorage.getItem('CDOPERADOR'),
        }).then((response: any) => {
            this.montaCard(response);
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        })
    }

    public montaCard(response: any) {
        const T = 'fm_00001_000_tarefas';
        const V = 'Detalhes_Vistoria';

        //this.dadosCard = [];

        if (response.data.length === 4 && response.data === 'NL{}') {
            const texto = 'Operador não cadastrado na hierarquia, favor verificar.';
            this.mensagem(texto);
        } else if (response.data.length === 1) {
            const texto = 'Não há ' + (response.data.find((n: string) => n === 'PAI') ? 'vistorias' : ' tarefas') + ' pendentes para este local';
            this.mensagem(texto);
        } else if (response.data.length === 2 && response.data.find((n: string) => n === 'PAI')) {
            const novaUrl = `${V}/${response.data[0].NRGERENCIAMENTOCHK}/${response.data[0].NMLOCALCLIENTE} | ${response.data[0].NMLOCALPAI} `;
            Router.replace(novaUrl);
            Utilities.showSuccessMessage();
            this.fecharModal();
        } else if (response.data.length === 2 && response.data.find((n: string) => n === 'FILHO')) {
            ///const novaUrl = (window.location.pathname.length >= 20) ? `${response.data[0].NRCHECKLISTAGEND}` :  `${T}/${response.data[0].NRCHECKLISTAGEND}`;
            Router.replace('/fm_00001_000_tarefas');
            this.telaTarefas.visibilidadePaginaInicial = false;
            this.telaTarefas.visibilidadeDetalhamento = true;
            this.telaTarefas.montarInformacoesComplementares(response.data[0].NRCHECKLISTAGEND);
            Utilities.showSuccessMessage();
            this.fecharModal();
        } else if (response.data.length > 2 && response.data.find((n: string) => n === 'PAI')) {
            const teste = '';
            this.fecharModal();
            Router.replace('Vistorias');
            //this.telaVistoria.filter(teste);
        } else if (response.data.length > 2 && response.data.find((n: string) => n === 'FILHO')) {
            //this.dadosCard = 
            this.fecharModal();
            this.abrirModalTarefas(response.data);
        }
    }

    public async abrirModalTarefas(dados: any) {
        if (!this.testeModal) {
            this.testeModal = await ModalService.createFromJson('modal_qr_code', true);    
        }
        this.testeModal.show();
        setTimeout(() => {
            this.montaGrid(dados);
        }, 3000);
    }

    public montaGrid(dados: any) {
        LoadingService.show();
        const grid = Metadata.getInstance<TekGrid>('tek-grid-escolha-tarefas');
        dados.forEach((dados: any) => {
            grid.datasource.data.push({
                NRCHECKLISTAGEND: dados.NRCHECKLISTAGEND,
                DATAHORA: dados.DATAHORA,
                NMCHECKLISTAGEND: dados.NMCHECKLISTAGEND,
            });
        });
        LoadingService.hide();
    }

    public async redirecionar({ row }: any) {
        Router.replace('/fm_00001_000_tarefas');
        LoadingService.show();
        setTimeout(async () => {
            this.telaTarefas.visibilidadePaginaInicial = false;
            this.telaTarefas.visibilidadeDetalhamento = true;
            this.telaTarefas.montarInformacoesComplementares(row.NRCHECKLISTAGEND);
        }, 3000);
        this.fecharModalTarefas();
        LoadingService.hide();
    } 

    public fecharModalTarefas() {
        if (this.testeModal) {
            this.testeModal.hide();
        }
    }

    public createModal() {
        const modal: IModal = {
            name: 'qrCodeModal',
            persistent: true,
            fullscreen: true,
            style: {
                background: '#black',
            },
            children: [
                {
                    name: 'closeButtonHeader',
                    component: 'ZdHeader',
                    color: 'transparent',
                    padless: true,
                    elevation: 0,
                    leftSlot: [
                        {
                            name: 'colTitleText',
                            component: 'ZdCol',
                            align: 'center',
                            cols: 12,
                            children: [
                                {
                                    name: 'titleText',
                                    component: 'ZdText',
                                    text: 'Escaneie o QR Code',
                                    cssClass: 'text-center',
                                    style: {
                                        'margin-bottom': '15px',
                                        'font-weight': 'bold',
                                        'font-size': '20px',
                                    },
                                },
                            ],
                        },
                    ],
                    rightSlot: [
                        {
                            name: 'closeModalButton',
                            component: 'ZdModalCloseButton',
                            small: true,
                            modalName: 'qrCodeModal',
                        },
                    ],
                },
                {
                    name: 'rowQrCode',
                    component: 'ZdRow',
                    children: [
                        {
                            name: 'qrcode',
                            component: 'QrCodeReader',
                            events: {
                                decode: this.decodeQrCode.bind(this),
                            },
                        },
                    ],
                },
            ],
        };
        this.qrCodeModal = ModalService.create(modal);
        this.qrCodeModal.show();
    }

    public fecharModal() {
        DialogService.hide();
        this.qrCodeModal.hide();
        this.qrCodeModal.destroy();
    }

    public mensagem(texto: string) {
        DialogService.show({
            name: 'basic-dialog',
            title: 'Atenção',
            text: `${texto}`,
            buttons: [
                {
                    name: 'yes',
                    component: 'ZdButton',
                    label: 'Sair',
                    events: {
                        click: '{{QrCodeController.fecharModal}}',
                    },
                },
            ],
        });
    }
}
